"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_extra_1 = require("fs-extra");
var handlebars_1 = require("handlebars");
var path_1 = require("path");
var process_1 = require("process");
var testTemplate = handlebars_1.compile(fs_extra_1.readFileSync(path_1.resolve(__dirname, "../../bin/test.template"), "utf8"));
exports.formatStep = function (step) {
    if (step.action === "scroll") {
        return "scroll " + step.scrollDirection;
    }
    var stepAction = step.action === "input" ? "enter" : "click";
    var stepValue = step.value ? "\"" + step.value + "\" into " : "";
    var targetTagName = step.target.inputType
        ? step.target.tagName + "[type=\"" + step.target.inputType + "\"]"
        : "" + (step.target.tagName === "a" ? "link" : step.target.tagName || "");
    var label = step.target.labels ? step.target.labels[0] : "";
    var targetName = label ||
        step.target.name ||
        step.target.placeholder ||
        step.target.textContent ||
        step.target.id ||
        "";
    var truncatedTargetName = targetName.trim().substring(0, 50);
    return stepAction + " " + stepValue + truncatedTargetName + " " + targetTagName;
};
exports.buildTest = function (job, useLocalModule) {
    if (useLocalModule === void 0) { useLocalModule = false; }
    var test = testTemplate({
        formattedSteps: job.steps.map(function (step) { return ({
            name: exports.formatStep(step),
            json: JSON.stringify(step)
        }); }),
        modulePath: useLocalModule ? process_1.cwd() + "/dist/index.js" : "qawolf",
        name: job.name,
        size: job.size,
        url: job.url
    });
    return test;
};
//# sourceMappingURL=buildTest.js.map