(function () {
    'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __awaiter(thisArg, _arguments, P, generator) {
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f) throw new TypeError("Generator is already executing.");
            while (_) try {
                if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
                if (y = 0, t) op = [op[0] & 2, t.value];
                switch (op[0]) {
                    case 0: case 1: t = op; break;
                    case 4: _.label++; return { value: op[1], done: false };
                    case 5: _.label++; y = op[1]; op = [0]; continue;
                    case 7: op = _.ops.pop(); _.trys.pop(); continue;
                    default:
                        if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                        if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                        if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                        if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                        if (t[2]) _.ops.pop();
                        _.trys.pop(); continue;
                }
                op = body.call(thisArg, _);
            } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
            if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
        }
    }

    var sleep = function (milliseconds) {
        return new Promise(function (resolve) { return setTimeout(resolve, milliseconds); });
    };
    var waitFor = function (valueFn, timeoutMs, sleepMs) {
        if (sleepMs === void 0) { sleepMs = 500; }
        return __awaiter(void 0, void 0, Promise, function () {
            var startTime, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        startTime = Date.now();
                        _a.label = 1;
                    case 1:
                        if (!(Date.now() - startTime < timeoutMs)) return [3, 3];
                        value = valueFn();
                        if (value)
                            return [2, value];
                        return [4, sleep(sleepMs)];
                    case 2:
                        _a.sent();
                        return [3, 1];
                    case 3: return [2, null];
                }
            });
        });
    };

    var scrollTo = function (yPosition, timeoutMs) {
        if (timeoutMs === void 0) { timeoutMs = 10000; }
        return __awaiter(void 0, void 0, void 0, function () {
            var start;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        start = Date.now();
                        _a.label = 1;
                    case 1:
                        if (!(window.pageYOffset !== yPosition)) return [3, 3];
                        if (Date.now() - start > timeoutMs) {
                            throw "Could not scroll, timeout exceeded";
                        }
                        return [4, sleep(100)];
                    case 2:
                        _a.sent();
                        window.scrollTo(0, yPosition);
                        return [3, 1];
                    case 3: return [2];
                }
            });
        });
    };

    var actions = /*#__PURE__*/Object.freeze({
        __proto__: null,
        scrollTo: scrollTo
    });

    var findElementByXpath = function (xpath) {
        var result = document.evaluate(xpath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if (result.singleNodeValue) {
            return result.singleNodeValue;
        }
        return null;
    };
    var buildXpath = function (element) {
        if (!element || element.nodeType !== 1)
            return "";
        if (element.id)
            return "//*[@id='" + element.id + "']";
        var children = element.parentNode ? element.parentNode.children : [];
        var sames = [].filter.call(children, function (x) {
            return x.tagName === element.tagName;
        });
        var result = buildXpath(element.parentNode) +
            "/" +
            element.tagName.toLowerCase() +
            (sames.length > 1
                ? "[" + ([].indexOf.call(sames, element) + 1) + "]"
                : "");
        return result;
    };
    var getXpath = function (element) {
        var result = buildXpath(element);
        return result
            .replace("svg", "*[name()='svg']")
            .replace("path", "*[name()='path']");
    };
    var isXpathEqual = function (xpathA, xpathB) {
        if (!xpathA || !xpathB)
            return false;
        if (xpathA === xpathB)
            return true;
        var resolveSameElement = findElementByXpath(xpathA) === findElementByXpath(xpathB);
        return resolveSameElement;
    };

    var xpath = /*#__PURE__*/Object.freeze({
        __proto__: null,
        findElementByXpath: findElementByXpath,
        getXpath: getXpath,
        isXpathEqual: isXpathEqual
    });

    var cleanText = function (text) {
        return text.trim().toLowerCase();
    };
    var getDataValue = function (element, dataAttribute) {
        if (!dataAttribute)
            return null;
        return element.getAttribute(dataAttribute) || null;
    };
    var getLabels = function (element) {
        var labelElements = element.labels;
        if (!labelElements || !labelElements.length)
            return null;
        var labels = [];
        for (var i = 0; i < labelElements.length; i++) {
            var textContent = labelElements[i].textContent;
            if (textContent) {
                labels.push(cleanText(textContent));
            }
        }
        return labels;
    };
    var getParentText = function (element) {
        if (!element.parentElement)
            return null;
        if (element.parentElement.textContent) {
            var parentText = [];
            for (var i = 0; i < element.parentElement.children.length; i++) {
                var textContent = element.parentElement.children[i].textContent;
                if (textContent) {
                    parentText.push(cleanText(textContent));
                }
                parentText.push(cleanText(element.parentElement.textContent));
            }
            return parentText;
        }
        return getParentText(element.parentElement);
    };
    var getPlaceholder = function (element) {
        if (element.tagName && element.tagName.toLowerCase() === "select") {
            var options = element.options;
            var placeholderOption = null;
            for (var i = 0; i < options.length; i++) {
                if (options[i].disabled) {
                    placeholderOption = options[i];
                    break;
                }
            }
            if (!placeholderOption || !placeholderOption.textContent)
                return null;
            return cleanText(placeholderOption.textContent);
        }
        else {
            if (!element.placeholder)
                return null;
            return cleanText(element.placeholder);
        }
    };
    var getTextContent = function (element) {
        if (!element.textContent)
            return null;
        return cleanText(element.textContent);
    };
    var getDescriptor = function (element, dataAttribute) {
        return {
            classList: (element.className || "").length
                ? element.className.split(" ")
                : null,
            dataValue: getDataValue(element, dataAttribute),
            href: element.href || null,
            id: element.id || null,
            inputType: element.type || null,
            labels: getLabels(element),
            name: element.name || null,
            parentText: getParentText(element),
            placeholder: getPlaceholder(element),
            tagName: element.tagName ? element.tagName.toLowerCase() : null,
            textContent: getTextContent(element),
            xpath: getXpath(element)
        };
    };

    var element = /*#__PURE__*/Object.freeze({
        __proto__: null,
        getDataValue: getDataValue,
        getLabels: getLabels,
        getParentText: getParentText,
        getPlaceholder: getPlaceholder,
        getTextContent: getTextContent,
        getDescriptor: getDescriptor
    });

    var strongMatchKeys = [
        "id",
        "name",
        "labels",
        "placeholder",
        "textContent",
        "xpath"
    ];
    var isNil = function (value) {
        return typeof value === "undefined" || value === null;
    };
    var compareArrays = function (base, compare) {
        if (!base || !base.length || !compare || !compare.length)
            return 0;
        var baseItemsInCompare = base.filter(function (v) { return compare.includes(v); }).length;
        return Math.round((baseItemsInCompare / base.length) * 100);
    };
    var compareDescriptorKey = function (key, targetValue, compareValue) {
        var percent;
        if (isNil(targetValue) || isNil(compareValue)) {
            percent = 0;
        }
        else if (key === "xpath") {
            var resolveSameElement = isXpathEqual(targetValue, compareValue);
            percent = resolveSameElement ? 100 : 0;
        }
        else if (Array.isArray(targetValue) && Array.isArray(compareValue)) {
            percent = compareArrays(targetValue, compareValue);
        }
        else {
            percent = targetValue === compareValue ? 100 : 0;
        }
        return { key: key, percent: percent };
    };
    var compareDescriptors = function (target, compare) {
        var matches = [];
        for (var _i = 0, _a = Object.keys(target); _i < _a.length; _i++) {
            var key = _a[_i];
            var match = compareDescriptorKey(key, target[key], compare[key]);
            if (match.percent > 0)
                matches.push(match);
        }
        return matches;
    };
    var matchElements = function (_a) {
        var dataAttribute = _a.dataAttribute, target = _a.target, elements = _a.elements, _b = _a.requireStrongMatch, requireStrongMatch = _b === void 0 ? false : _b;
        var matches = [];
        elements.forEach(function (element) {
            var descriptor = getDescriptor(element, dataAttribute);
            var targetMatches = compareDescriptors(target, descriptor);
            var strongMatch = targetMatches.find(function (m) {
                return strongMatchKeys.includes(m.key);
            });
            var majorityMatch = targetMatches.length / Object.keys(target).length > 0.5;
            if (!strongMatch && !majorityMatch) {
                targetMatches = [];
            }
            else if (requireStrongMatch && !strongMatch) {
                targetMatches = [];
            }
            var value = targetMatches.reduce(function (accumulator, match) { return accumulator + match.percent; }, 0);
            if (value) {
                matches.push({ element: element, targetMatches: targetMatches, value: value });
            }
        });
        return matches;
    };
    var topMatch = function (args) {
        var matches = matchElements(args).sort(function (a, b) {
            return a.value > b.value ? -1 : 1;
        });
        if (matches.length <= 0 || matches[0].value <= 0) {
            console.log("no match found");
            return null;
        }
        var equalTopMatches = matches.filter(function (m) { return m.value === matches[0].value; });
        if (equalTopMatches.length > 1) {
            console.log("no match since all top matches are equal", equalTopMatches);
            return null;
        }
        console.log("top match found", matches[0]);
        return matches[0];
    };

    var match = /*#__PURE__*/Object.freeze({
        __proto__: null,
        compareArrays: compareArrays,
        compareDescriptorKey: compareDescriptorKey,
        compareDescriptors: compareDescriptors,
        matchElements: matchElements,
        topMatch: topMatch
    });

    var isVisible = function (element) {
        return !!(element.offsetWidth && element.offsetHeight);
    };
    var queryActionElements = function (action) {
        var selector = action === "input" ? "input,select,textarea" : "*";
        return queryVisibleElements(selector);
    };
    var queryDataElements = function (_a) {
        var action = _a.action, dataAttribute = _a.dataAttribute, dataValue = _a.dataValue;
        var dataSelector = "[" + dataAttribute + "='" + dataValue + "']";
        if (action === "input") {
            var selector = "input" + dataSelector + ",select" + dataSelector + ",textarea" + dataSelector;
            return queryVisibleElements(selector);
        }
        return queryVisibleElements(dataSelector);
    };
    var queryVisibleElements = function (selector) {
        var elements = document.querySelectorAll(selector);
        var visibleElements = [];
        for (var i = 0; i < elements.length; i++) {
            if (isVisible(elements[i])) {
                visibleElements.push(elements[i]);
            }
        }
        return visibleElements;
    };
    var waitForElement = function (_a) {
        var action = _a.action, dataAttribute = _a.dataAttribute, target = _a.target, timeoutMs = _a.timeoutMs;
        return __awaiter(void 0, void 0, void 0, function () {
            var strongMatch, elements, match;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (dataAttribute && target.dataValue) {
                            console.log("finding element by data attribute " + dataAttribute + "=" + target.dataValue);
                            return [2, waitFor(function () {
                                    var elements = queryDataElements({
                                        action: action,
                                        dataAttribute: dataAttribute,
                                        dataValue: target.dataValue
                                    });
                                    var match = topMatch({ dataAttribute: dataAttribute, target: target, elements: elements });
                                    if (match)
                                        return match.element;
                                    return null;
                                }, timeoutMs)];
                        }
                        console.log("waiting for strong match");
                        return [4, waitFor(function () {
                                var elements = queryActionElements(action);
                                return topMatch({
                                    dataAttribute: dataAttribute,
                                    target: target,
                                    elements: elements,
                                    requireStrongMatch: true
                                });
                            }, timeoutMs)];
                    case 1:
                        strongMatch = _b.sent();
                        if (strongMatch)
                            return [2, strongMatch.element];
                        console.log("no strong match found before timeout, choosing top weak match");
                        elements = queryActionElements(action);
                        match = topMatch({ dataAttribute: dataAttribute, target: target, elements: elements });
                        if (match)
                            return [2, match.element];
                        return [2, null];
                }
            });
        });
    };

    var locate = /*#__PURE__*/Object.freeze({
        __proto__: null,
        isVisible: isVisible,
        queryActionElements: queryActionElements,
        queryDataElements: queryDataElements,
        queryVisibleElements: queryVisibleElements,
        waitForElement: waitForElement
    });

    var qawolf = {
        actions: actions,
        element: element,
        locate: locate,
        match: match,
        xpath: xpath
    };
    if (typeof window !== "undefined" && !window.qawolf) {
        window.qawolf = qawolf;
        console.log("loaded qawolf");
    }

}());
