"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var xpath_1 = require("./xpath");
var element_1 = require("./element");
var isNil = function (value) {
    return typeof value === "undefined" || value === null;
};
exports.buildDescriptorMatches = function (target, compare) {
    var matches = [];
    for (var _i = 0, _a = Object.keys(target); _i < _a.length; _i++) {
        var key = _a[_i];
        var targetValue = target[key];
        var compareValue = compare[key];
        if (isNil(targetValue) || isNil(compareValue))
            continue;
        var percent = 0;
        if (key === "xpath") {
            percent = xpath_1.isXpathEqual(targetValue, compareValue)
                ? 100
                : 0;
        }
        else if (Array.isArray(targetValue) && Array.isArray(compareValue)) {
            percent = exports.compareArray(targetValue, compareValue);
        }
        else {
            percent = targetValue === compare[key] ? 100 : 0;
        }
        if (percent > 0) {
            matches.push({ key: key, percent: percent });
        }
    }
    return matches;
};
exports.compareArray = function (base, compare) {
    if (!base || !base.length || !compare || !compare.length)
        return 0;
    var baseItemsInCompare = base.filter(function (v) { return compare.includes(v); }).length;
    return Math.round((baseItemsInCompare / base.length) * 100);
};
var strongMatchKeys = [
    "id",
    "labels",
    "name",
    "placeholder",
    "textContent",
    "xpath"
];
exports.topMatchElement = function (_a) {
    var dataAttribute = _a.dataAttribute, target = _a.target, elements = _a.elements, _b = _a.requireSpecific, requireSpecific = _b === void 0 ? false : _b;
    var topScore = -1;
    var topElement = null;
    elements.forEach(function (element) {
        var descriptor = element_1.getDescriptor(element, dataAttribute);
        var matches = exports.buildDescriptorMatches(target, descriptor);
        if (requireSpecific &&
            !matches.find(function (m) { return strongMatchKeys.includes(m.key); })) {
            return;
        }
        var matchValue = matches.reduce(function (accumulator, match) { return accumulator + match.percent; }, 0);
        if (matchValue > topScore) {
            topElement = element;
        }
        else if (matchValue === topScore) {
            topElement = null;
        }
    });
    return topElement;
};
//# sourceMappingURL=compare.js.map