"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var Browser_1 = require("../browser/Browser");
var config_1 = require("../config");
var match_1 = require("./match");
describe("compareArrays", function () {
    test("returns 0 if either array is null", function () {
        expect(match_1.compareArrays(null, ["spirit"])).toBe(0);
        expect(match_1.compareArrays(["spirit"], null)).toBe(0);
    });
    test("returns 0 if either array is empty", function () {
        expect(match_1.compareArrays([], ["spirit"])).toBe(0);
        expect(match_1.compareArrays(["spirit"], [])).toBe(0);
    });
    test("returns the share of base items in compare", function () {
        expect(match_1.compareArrays(["spirit", "bobcat"], ["spirit", "bear"])).toBe(50);
        expect(match_1.compareArrays(["spirit", "bobcat"], ["spirit", "spirit"])).toBe(50);
    });
});
describe("compareDescriptorKey", function () {
    test("returns 0 if either value is falsy", function () {
        expect(match_1.compareDescriptorKey("id", null, "spirit")).toEqual({
            key: "id",
            percent: 0
        });
        expect(match_1.compareDescriptorKey("id", "spirit", null)).toEqual({
            key: "id",
            percent: 0
        });
    });
    test("returns 0 if different xpaths", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            return qawolf.match.compareDescriptorKey("xpath", "//*[@id='username']", "//*[@id='password']");
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toEqual({ key: "xpath", percent: 0 });
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    test("returns 100 if same xpaths", function () {
        expect(match_1.compareDescriptorKey("xpath", "//*[@id='password']", "//*[@id='password']")).toEqual({ key: "xpath", percent: 100 });
    });
    test("returns share of base items in compare if arrays", function () {
        expect(match_1.compareDescriptorKey("labels", ["spirit", "bobcat"], ["spirit", "bear"])).toEqual({ key: "labels", percent: 50 });
    });
    test("returns 0 if strings not equal", function () {
        expect(match_1.compareDescriptorKey("id", "spirit", "bobcat")).toEqual({
            key: "id",
            percent: 0
        });
    });
    test("returns 100 if strings equal", function () {
        expect(match_1.compareDescriptorKey("id", "spirit", "spirit")).toEqual({
            key: "id",
            percent: 100
        });
    });
});
describe("compareDescriptors", function () {
    test("returns list of matches between target and compare", function () {
        expect(match_1.compareDescriptors({
            id: "spirit",
            labels: ["spirit", "wolf"],
            tagName: "div",
            textContent: "spirit"
        }, { id: "bobcat", labels: ["spirit"], tagName: "div" })).toEqual([
            { key: "labels", percent: 50 },
            { key: "tagName", percent: 100 }
        ]);
    });
    test("returns empty array if no matches", function () {
        expect(match_1.compareDescriptors({
            id: "spirit",
            labels: ["spirit"],
            textContent: "spirit"
        }, { id: "bobcat", labels: ["bobcat"], tagName: "div" })).toEqual([]);
    });
});
describe("matchElements", function () {
    test("returns elements that match if data attribute specified", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            var username = document.getElementById("username");
                            username.setAttribute("data-qa", "username");
                            var password = document.getElementById("password");
                            password.setAttribute("data-qa", "username");
                            var collection = document.querySelectorAll("input,button");
                            var elements = [];
                            for (var i = 0; i < collection.length; i++) {
                                elements.push(collection[i]);
                            }
                            var matches = qawolf.match.matchElements({
                                dataAttribute: "data-qa",
                                target: {
                                    dataValue: "username",
                                    labels: ["username"],
                                    tagName: "input"
                                },
                                elements: elements
                            });
                            username.removeAttribute("data-qa");
                            password.removeAttribute("data-qa");
                            return matches.map(function (match) {
                                return {
                                    element: qawolf.xpath.getXpath(match.element),
                                    targetMatches: match.targetMatches,
                                    value: match.value
                                };
                            });
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toEqual([
                        {
                            element: "//*[@id='username']",
                            targetMatches: [
                                { key: "dataValue", percent: 100 },
                                { key: "labels", percent: 100 },
                                { key: "tagName", percent: 100 }
                            ],
                            value: 300
                        },
                        {
                            element: "//*[@id='password']",
                            targetMatches: [
                                { key: "dataValue", percent: 100 },
                                { key: "tagName", percent: 100 }
                            ],
                            value: 200
                        }
                    ]);
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    test("returns elements that match if data attribute not specified", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            var collection = document.querySelectorAll("input,button");
                            var elements = [];
                            for (var i = 0; i < collection.length; i++) {
                                elements.push(collection[i]);
                            }
                            var matches = qawolf.match.matchElements({
                                dataAttribute: null,
                                target: { labels: ["username"], tagName: "input" },
                                elements: elements
                            });
                            return matches.map(function (match) {
                                return {
                                    element: qawolf.xpath.getXpath(match.element),
                                    targetMatches: match.targetMatches,
                                    value: match.value
                                };
                            });
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toEqual([
                        {
                            element: "//*[@id='username']",
                            targetMatches: [
                                { key: "labels", percent: 100 },
                                { key: "tagName", percent: 100 }
                            ],
                            value: 200
                        }
                    ]);
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    test("returns empty array if strong matches required and none found", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            var collection = document.querySelectorAll("input,button");
                            var elements = [];
                            for (var i = 0; i < collection.length; i++) {
                                elements.push(collection[i]);
                            }
                            var matches = qawolf.match.matchElements({
                                dataAttribute: null,
                                target: { tagName: "input" },
                                elements: elements,
                                requireStrongMatch: true
                            });
                            return matches.map(function (match) {
                                return {
                                    element: qawolf.xpath.getXpath(match.element),
                                    targetMatches: match.targetMatches,
                                    value: match.value
                                };
                            });
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toEqual([]);
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
});
describe("topMatch", function () {
    test("returns null if no matches found", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            var collection = document.querySelectorAll("input,button");
                            var elements = [];
                            for (var i = 0; i < collection.length; i++) {
                                elements.push(collection[i]);
                            }
                            var match = qawolf.match.topMatch({
                                dataAttribute: null,
                                target: { tagName: "input" },
                                elements: elements,
                                requireStrongMatch: true
                            });
                            if (!match)
                                return null;
                            return {
                                element: qawolf.xpath.getXpath(match.element),
                                targetMatches: match.targetMatches,
                                value: match.value
                            };
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toBeNull();
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    test("returns null if multiple tied matches found", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            var collection = document.querySelectorAll("input,button");
                            var elements = [];
                            for (var i = 0; i < collection.length; i++) {
                                elements.push(collection[i]);
                            }
                            var match = qawolf.match.topMatch({
                                dataAttribute: null,
                                target: { tagName: "input" },
                                elements: elements
                            });
                            if (!match)
                                return null;
                            return {
                                element: qawolf.xpath.getXpath(match.element),
                                targetMatches: match.targetMatches,
                                value: match.value
                            };
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toBeNull();
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    test("returns top match if one found", function () { return __awaiter(void 0, void 0, void 0, function () {
        var browser, page, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, Browser_1.Browser.create({ url: config_1.CONFIG.testUrl + "login" })];
                case 1:
                    browser = _a.sent();
                    return [4, browser.currentPage()];
                case 2:
                    page = _a.sent();
                    return [4, page.evaluate(function () {
                            var qawolf = window.qawolf;
                            var collection = document.querySelectorAll("input,button");
                            var elements = [];
                            for (var i = 0; i < collection.length; i++) {
                                elements.push(collection[i]);
                            }
                            var match = qawolf.match.topMatch({
                                dataAttribute: null,
                                target: { labels: ["username"], tagName: "input" },
                                elements: elements
                            });
                            if (!match)
                                return null;
                            return {
                                element: qawolf.xpath.getXpath(match.element),
                                targetMatches: match.targetMatches,
                                value: match.value
                            };
                        })];
                case 3:
                    result = _a.sent();
                    expect(result).toEqual({
                        element: "//*[@id='username']",
                        targetMatches: [
                            { key: "labels", percent: 100 },
                            { key: "tagName", percent: 100 }
                        ],
                        value: 200
                    });
                    return [4, browser.close()];
                case 4:
                    _a.sent();
                    return [2];
            }
        });
    }); });
});
//# sourceMappingURL=match.test.js.map