"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var xpath_1 = require("./xpath");
exports.computeArraySimilarityScore = function (compare, base) {
    if (!base || !compare || !base.length)
        return 0;
    var similarityScore = 0;
    base.forEach(function (value) {
        if (compare.includes(value)) {
            similarityScore += 1;
        }
    });
    return Math.round((similarityScore / base.length) * 100);
};
exports.computeStringSimilarityScore = function (compare, base) {
    if (!base || !compare)
        return 0;
    if (base === compare)
        return 100;
    return 0;
};
exports.computeSimilarityScore = function (compare, base) {
    var score = 0;
    score += exports.computeArraySimilarityScore(compare.classList, base.classList);
    score += exports.computeStringSimilarityScore(compare.dataValue, base.dataValue);
    score += exports.computeStringSimilarityScore(compare.href, base.href);
    score += exports.computeStringSimilarityScore(compare.id, base.id);
    score += exports.computeStringSimilarityScore(compare.inputType, base.inputType);
    score += exports.computeArraySimilarityScore(compare.labels, base.labels);
    score += exports.computeStringSimilarityScore(compare.name, base.name);
    score += exports.computeArraySimilarityScore(compare.parentText, base.parentText);
    score += exports.computeStringSimilarityScore(compare.placeholder, base.placeholder);
    score += exports.computeStringSimilarityScore(compare.tagName, base.tagName);
    score += exports.computeStringSimilarityScore(compare.textContent, base.textContent);
    score += xpath_1.compareXpaths(compare.xpath, base.xpath) ? 100 : 0;
    return score;
};
exports.computeMaxPossibleScore = function (base) {
    var score = 0;
    Object.values(base).forEach(function (value) {
        if (value !== null) {
            score += 100;
        }
    });
    if (!score) {
        throw "Base element " + base + " has no recorded properties";
    }
    return score;
};
//# sourceMappingURL=score.js.map