"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var score_1 = require("./score");
var base = {
    classList: ["spirit", "bobcat", "moose"],
    href: null,
    id: "spirit",
    inputType: "submit",
    labels: null,
    name: null,
    parentText: ["some text", "even more text"],
    placeholder: null,
    tagName: "input",
    textContent: "click me!"
};
describe("score.computeArraySimilarityScore", function () {
    test("returns 0 if either are null", function () {
        var score = score_1.computeArraySimilarityScore(null, null);
        expect(score).toBe(0);
        var score2 = score_1.computeArraySimilarityScore(["spirit"], null);
        expect(score2).toBe(0);
        var score3 = score_1.computeArraySimilarityScore(null, ["spirit"]);
        expect(score3).toBe(0);
    });
    test("returns 0 if base array empty", function () {
        var score = score_1.computeArraySimilarityScore(["spirit"], []);
        expect(score).toBe(0);
    });
    test("returns rounded percent similar if both have items", function () {
        var score = score_1.computeArraySimilarityScore(["spirit", "bobcat", "bear"], ["spirit", "bobcat", "moose"]);
        expect(score).toBe(67);
    });
});
describe("score.computeStringSimilarityScore", function () {
    test("returns 0 if either are null", function () {
        var score = score_1.computeStringSimilarityScore(null, null);
        expect(score).toBe(0);
        var score2 = score_1.computeStringSimilarityScore("spirit", null);
        expect(score2).toBe(0);
        var score3 = score_1.computeStringSimilarityScore(null, "spirit");
        expect(score3).toBe(0);
    });
    test("returns 100 if strings are the same", function () {
        var score = score_1.computeStringSimilarityScore("spirit", "spirit");
        expect(score).toBe(100);
    });
    test("returns 0 if strings are different", function () {
        var score = score_1.computeStringSimilarityScore("spirit", "bobcat");
        expect(score).toBe(0);
    });
});
describe("score.computeSimilarityScore", function () {
    test("returns correct similarity score", function () {
        var compare = {
            classList: ["spirit", "bobcat", "bear"],
            href: null,
            id: "spirit",
            inputType: null,
            labels: null,
            name: null,
            parentText: ["some text", "more text"],
            placeholder: null,
            tagName: "button",
            textContent: "click me!"
        };
        var score = score_1.computeSimilarityScore(compare, base);
        expect(score).toBe(317);
    });
    test("returns 100 if xpaths are the same", function () {
        var score = score_1.computeSimilarityScore({ xpath: '//*[@id="spirit"]' }, { xpath: '//*[@id="spirit"]' });
        expect(score).toBe(100);
    });
});
describe("score.computeMaxPossibleScore", function () {
    test("returns correct max possible score", function () {
        var score = score_1.computeMaxPossibleScore(base);
        expect(score).toBe(600);
    });
    test("throws error if no properties recorded", function () {
        var baseEmpty = {
            classList: null,
            href: null,
            id: null,
            inputType: null,
            labels: null,
            name: null,
            parentText: null,
            placeholder: null,
            tagName: null,
            textContent: null
        };
        expect(function () { return score_1.computeMaxPossibleScore(baseEmpty); }).toThrowError();
    });
});
//# sourceMappingURL=score.test.js.map